/**
 Trivial multiple-processes program,
 showing how to use pipes for processes to communicate.
 
 Orion Sky Lawlor, olawlor@acm.org, 2005/2/23 (Public Domain)
*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h> /* for fork */
#include <sys/wait.h>


/* File descriptors returned by "pipe" system call. 
  [0] is the read end of the pipe (outflow)
  [1] is the write end of the pipe (inflow)
*/
int pipeEnds[2];

void *runA(void *) {
	for (int n=0;n<20;n++) {
		for (int i=0;i<1000*1000;i++) {}
		if (n==10) {
			printf("A: Receiving data from pipe\n");
			int len;
			read(pipeEnds[0],&len,sizeof(len));
			char *buf=new char[len];
			read(pipeEnds[0],buf,len);
			printf("A: Data received: '%s'\n",buf);
			delete[] buf;
		}
		printf("A (%d)\n",n);
	}
}

void *runB(void *) {
	for (int n=0;n<20;n++) {
		for (int i=0;i<1000*1000;i++) {}
		if (n==15) {
			printf("      B: Sending data over pipe\n");
			char buf[100];
			sprintf(buf,"Hi!  I'm on iteration %d!",n);
			int len=strlen(buf)+1;
			write(pipeEnds[1],&len,sizeof(len));
			write(pipeEnds[1],buf,len);
			printf("      B: Data sent.\n");
		}
		printf("      B (%d)\n",n);
	}
}

int main() {
	pipe(pipeEnds);
	if (fork()==0) { /* child: do A work */
		runA(0);
	} else { /* parent: do B work */
		runB(0);
		wait((int *)0);
	}
	return 0;
}
